<?php

namespace Drupal\Tests\acquia_dam\FunctionalJavascript;

/**
 * Tests image render using 'Thumbnail' field formatter of core.
 *
 * This module alter the definition of 'Thumbnail' formatter and
 * provides its own formatter to handle the rendering functionality.
 *
 * @see acquia_dam_field_formatter_info_alter
 *
 * @group acquia_dam
 */
final class RenderImageWithImageStyleTest extends AcquiaDamWebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->createMediaReferenceField(1);
  }

  /**
   * Creating a media field for the test.
   */
  protected function createMediaReferenceField($cardinality = -1) {
    $this->createEntityReferenceField(
      'node',
      'page',
      'field_acquia_dam',
      'Acquia DAM',
      'media',
      'default:media',
      [],
      $cardinality,
    );
    $display_repository = $this->container->get('entity_display.repository');
    $display_repository->getFormDisplay('node', 'page')
      ->setComponent('field_acquia_dam', [
        'type' => 'media_library_widget',
        'region' => 'content',
      ])->save();
    $display_repository->getViewDisplay('node', 'page', 'default')
      ->setComponent('field_acquia_dam', [
        'type' => 'media_thumbnail',
      ])->save();
  }

  /**
   * Update field display with given image style.
   *
   * @param string $image_style
   *   The image style.
   */
  private function updateFieldViewDisplay(string $image_style): void {
    $this->container->get('entity_display.repository')
      ->getViewDisplay('node', 'page', 'default')
      ->setComponent('field_acquia_dam', [
        'type' => 'media_thumbnail',
        'settings' => [
          'image_link' => '',
          'image_style' => $image_style,
          'image_loading' => ['attribute' => 'lazy'],
        ],
      ])->save();
  }

  /**
   * Test image rendered using selected image style.
   *
   * @param string $image_style
   *   The image styles.
   * @param string $asset_id
   *   The acquia dam asset ID.
   * @param string $asset_version
   *   Acquia dam assert version.
   *
   * @dataProvider imageStyleProvider
   */
  public function testImageWithImageStyle(string $image_style, string $asset_id, string $asset_version) {
    // Update field view display settings with selected image style.
    $this->updateFieldViewDisplay($image_style);

    // Create user and login.
    $user = $this->createUser(['create page content', 'edit any page content']);
    $this->drupalLogin($user);

    // Acquia DAM authorization.
    $this->grantSiteRegistrationToken();
    $this->grantCurrentUserDamToken();

    // Create node of type page and add media from acquia DAM.
    $this->drupalGet('node/add/page');
    $this->getSession()->getPage()->fillField('Title', 'Image with image style large');
    $wrapper = $this->assertSession()->elementExists('css', '#field_acquia_dam-media-library-wrapper');
    $wrapper->pressButton('Add media');
    $this->assertNotNull($this->assertSession()->waitForText('Select Media Source'));

    // Wait for media library modal to open.
    $modal = $this->assertSession()->waitForElement('css', '#drupal-modal');
    $modal->clickLink('Image');
    $this->assertSession()->assertWaitOnAjaxRequest();

    // Select and insert specified image.
    $this->selectAndInsertAsset($asset_id, 'Image', '', FALSE);

    $this->getSession()->switchToIFrame();
    $this->getSession()->getPage()->pressButton('Save');

    $this->getSession()->getPage()->findLink('Image with image style large')->click();

    // Validate image rendered with selected images style.
    $expected_uri = "styles/$image_style/public/acquia_dam_asset_files/$asset_id/$asset_version";
    $this->assertSession()->elementAttributeContains('css', 'img', 'src', $expected_uri);
    $this->assertSession()->elementAttributeContains('css', 'img', 'loading', 'lazy');
  }

  /**
   * Image style provider.
   */
  public static function imageStyleProvider(): array {
    return [
      ['large', 'cd3d3d5e-6ad2-4a6c-ba76-a6fb516e367b', '9e4e810c-147b-4ac2-85a9-cf64f8fa61e0'],
      ['media_library', 'cd3d3d5e-6ad2-4a6c-ba76-a6fb516e367b', '9e4e810c-147b-4ac2-85a9-cf64f8fa61e0'],
      ['medium', 'cd3d3d5e-6ad2-4a6c-ba76-a6fb516e367b', '9e4e810c-147b-4ac2-85a9-cf64f8fa61e0'],
      ['thumbnail', 'cd3d3d5e-6ad2-4a6c-ba76-a6fb516e367b', '9e4e810c-147b-4ac2-85a9-cf64f8fa61e0'],
      ['wide', 'cd3d3d5e-6ad2-4a6c-ba76-a6fb516e367b', '9e4e810c-147b-4ac2-85a9-cf64f8fa61e0'],
    ];
  }

}
